#ifndef _RayTracer_h_
#define _RayTracer_h_

#include "Vertex.h"
#include <vector>

namespace RayTracer {
	struct Image {
		std::vector<unsigned char> data;
		int width;
		int height;
		int bpp;
	};

	enum ObjectType {
		Object_Sphere,
		Object_Triangle,
	};

	struct Object {
		ObjectType type;
		Vertex<3> ambient;
		Vertex<3> diffuse;
		Vertex<3> specular;
		Vertex<3> reflective;
		Vertex<3> transmittance;
		double specularExponent;
		double refractiveIndex;
		int texture;

		union {
			struct {
				Vertex<3> center;
				double radius;
			} sphere;
			struct {
				Vertex<3> vertex[3];
				Vertex<3> normal;
			} triangle;
		} object;
	};

	struct Light {
		Vertex<3> position;
		Vertex<3> color;
	};

	Image render(Vertex<3> cameraPosition, Vertex<3> lookAt, Vertex<3> up,
		double fovy, double aspect, int imageSize[2], Vertex<3> ambient,
		Vertex<3> backgroundColor, double defaultRefractiveIndex,
		const std::vector<Object> &objects, const std::vector<Light> &lights,
		int maxRecursion, int oversample, int debugx = -1, int debugy = -1);

	Vertex<3> traceRay(const Vertex<3> &origin, const Vertex<3> &direction,
		const std::vector<Object> &objects, const std::vector<Light> &lights,
		int ttl, double refractiveIndex, const Vertex<3> &globalAmbient,
		const Vertex<3> &backgroundColor, double defaultRefractiveIndex);
};

#endif
