#ifndef _Vertex_h_
#define _Vertex_h_

#include <cmath>
#include <iostream>

template<int S> struct Vertex {
	double v[S];
};

template<int S> Vertex<S> operator-(const Vertex<S> &a, const Vertex<S> &b) {
	Vertex<S> result;

	for (int i = 0; i < S; i++) {
		result.v[i] = a.v[i] - b.v[i];
	}

	return result;
}

template<int S> Vertex<S> operator+(const Vertex<S> &a, const Vertex<S> &b) {
	Vertex<S> result;

	for (int i = 0; i < S; i++) {
		result.v[i] = a.v[i] + b.v[i];
	}

	return result;
}

template<int S> Vertex<S> operator*(const Vertex<S> &a, double b) {
	Vertex<S> result;

	for (int i = 0; i < S; i++) {
		result.v[i] = a.v[i] * b;
	}

	return result;
}

template<int S> Vertex<S> operator/(const Vertex<S> &a, double b) {
	Vertex<S> result;

	for (int i = 0; i < S; i++) {
		result.v[i] = a.v[i] / b;
	}

	return result;
}

template<int S> Vertex<S> normalize(const Vertex<S> &a) {
	Vertex<S> result;
	double magnitude = 0.0;

	for (int i = 0; i < S; i++) {
		magnitude += a.v[i] * a.v[i];
	}

	magnitude = sqrt(magnitude);

	for (int i = 0; i < S; i++) {
		result.v[i] = a.v[i] / magnitude;
	}

	return result;
}

template<int S> double dotProduct(const Vertex<S> &a, const Vertex<S> &b) {
	double dot = 0.0;

	for (int i = 0; i < S; i++) {
		dot += a.v[i] * b.v[i];
	}

	return dot;
}

template<int S> Vertex<S> memberwiseProduct(const Vertex<S> &a, const Vertex<S> &b) {
	Vertex<S> product;

	for (int i = 0; i < S; i++) {
		product.v[i] = a.v[i] * b.v[i];
	}

	return product;
}

template<int S> double abs(const Vertex<S> &a) {
	return sqrt(dotProduct(a, a));
}

template<int S> std::ostream &operator<<(std::ostream &stream, const Vertex<S> &a) {
	std::cout << "<";
	for (int i = 0; i < S; i++) {
		std::cout << a.v[i];
		if (i != S - 1) {
			std::cout << ", ";
		}
	}
	return stream << ">";
}

Vertex<3> crossProduct(const Vertex<3> &a, const Vertex<3> &b);

#endif
